@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free
-  Software Foundation, Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
+  2015 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -262,10 +262,19 @@ setpeer (int argc, char **argv)
 
   if (argc == 3)
     {
-      port = atoi (argv[2]);
+      if (isdigit(argv[2][0]) || argv[2][0] == '-')
+	port = atoi (argv[2]);
+      else
+	{
+	  struct servent *sp;
+
+	  sp = getservbyname (argv[2], "tcp");
+	  port = (sp) ? ntohs (sp->s_port) : 0;
+	}
+
       if (port <= 0 || port > 65535)
 	{
-	  printf ("%s: bad port number-- %s\n", argv[1], argv[2]);
+	  printf ("%s: bad port -- %s\n", argv[1], argv[2]);
 	  printf ("usage: %s host-name [port]\n", argv[0]);
 	  code = -1;
 	  return;
@@ -279,8 +288,14 @@ setpeer (int argc, char **argv)
       port = (sp) ? ntohs (sp->s_port) : DEFPORT;
     }
 
-  host = hookup (host, port);
-  if (host)
+  /* After hookup(), the global variable `hostname' contains
+   * the canonical host name corresponding to the alias name
+   * contained in HOST.  The return value of hookup() is not
+   * NULL only if the server has answered our call.  The value
+   * of HOST should be preserved for reporting inside login(),
+   * which also detects a correct stanza of the netrc file.
+   */
+  if (hookup (host, port))
     {
       int overbose;
 
@@ -348,7 +363,7 @@ setpeer (int argc, char **argv)
 	    unix_server = 0;
 	  if (overbose && !strncmp (reply_string, "215 TOPS20", 10))
 	    printf
-	      ("Remember to set tenex mode when transfering binary files from this machine.\n");
+	      ("Remember to set tenex mode when transferring binary files from this machine.\n");
 	}
       verbose = overbose;
 #endif /* (unix || __unix || __unix__) && (NBBY == 8) */
@@ -1198,20 +1213,57 @@ settrace (int argc _GL_UNUSED_PARAMETER, char **argv _GL_UNUSED_PARAMETER)
 
 /*
  * Toggle hash mark printing during transfers.
+ *
+ * Parse multipliers 'k', 'K', 'm', 'M', and
+ * 'g', 'G' to simplify the size step.
+ *
+ * With a numerical argument, hash marking is
+ * made active, and the step size is updated.
+ * Toggle state only in absence of an argument.
  */
 void
 sethash (int argc _GL_UNUSED_PARAMETER, char **argv)
 {
+  char *p = argv[1];
+
+  /* P is NULL when no argument was passed with `hash'.  */
+  while (p && isdigit (*p))
+    p++;
+
   if (argv[1] != NULL)
     sscanf (argv[1], "%d", &hashbytes);
+
+  /* Apply a multiplier only if a numerical part exists.  */
+  if (argv[1] && isdigit (*argv[1]))
+    {
+      hash = 1;			/* Enforce markers.  */
+
+      switch (*p)
+	{
+	case 'g':
+	case 'G':
+	  hashbytes *= 1024;	/* Cascaded multiplication!  */
+	case 'm':
+	case 'M':
+	  hashbytes *= 1024;
+	case 'k':
+	case 'K':
+	  hashbytes *= 1024;
+	}
+    }
+
   if (hashbytes <= 0)
     hashbytes = 1024;
-  hash = !hash;
+
+  if (!argv[1])			/* Toggle when argument is absent.  */
+    hash = !hash;
+
   printf ("Hash mark printing %s", onoff (hash));
-  code = hash;
   if (hash)
     printf (" (%d bytes/hash mark)", hashbytes);
   printf (".\n");
+
+  code = hash;
 }
 
 /*
@@ -1683,8 +1735,20 @@ user (int argc, char **argv)
   n = command ("USER %s", argv[1]);
   if (n == CONTINUE)
     {
+      /* Is this a case of challenge-response?
+       * RFC 2228 stipulates code 336 for this.
+       * Suppress message in verbose mode, since
+       * it has already been displayed.
+       */
+      if (code == 336 && !verbose)
+	printf ("%s\n", reply_string + strlen ("336 "));
+      /* In addition, any password given on the
+       * command line is irrelevant, so ignore it.
+       */
+      if (argc < 3 || code == 336)
+	argv[2] = getpass ("Password: ");
       if (argc < 3)
-	argv[2] = getpass ("Password: "), argc++;
+	argc++;
       n = command ("PASS %s", argv[2]);
       if (argv[2])
 	memset (argv[2], 0, strlen (argv[2]));
