@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free
-  Software Foundation, Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
+  2015 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -123,6 +123,14 @@ remote_userpass (char *host, char **aname, char **apass, char **aacct)
   if (hdir == NULL)
     hdir = ".";
   snprintf (buf, sizeof buf, "%s/.netrc", hdir);
+
+  /* The switch `-N/--netrc' would have set this.  */
+  if (!netrc)
+    netrc = getenv ("NETRC");
+
+  if (netrc && netrc[0])
+    snprintf (buf, sizeof buf, "%s", netrc);
+
   cfile = fopen (buf, "r");
   if (cfile == NULL)
     {
@@ -131,6 +139,28 @@ remote_userpass (char *host, char **aname, char **apass, char **aacct)
       return (0);
     }
 
+  /* The .netrc is now opened and is thus fixed.
+   * Check that it is a regular file, and not a
+   * soft link in particular.
+   */
+  if (lstat (buf, &stb) < 0)
+    {
+      error (0, errno, "%s", buf);
+      fclose (cfile);
+      return (-1);
+    }
+
+  if (!S_ISREG (stb.st_mode))
+    {
+      if (S_ISLNK (stb.st_mode))
+	error (0, 0, "the .netrc file is symbolic link: %s", buf);
+      else
+	error (0, 0, "the .netrc file is no regular file: %s", buf);
+
+      fclose (cfile);
+      return (-1);
+    }
+
   myname = localhost ();
   if (!myname)
     myname = xstrdup ("");
