@@ -1,6 +1,6 @@
 /* bsd.c -- BSD specific code for ifconfig
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011, 2012, 2013 Free Software Foundation, Inc.
+  2010, 2011, 2012, 2013, 2014, 2015 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -60,15 +60,17 @@ system_parse_opt (struct ifconfig **ifp _GL_UNUSED_PARAMETER,
 int
 system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
 {
-  int i = 0;
+  int i = 0, mask, rev;
   enum
   {
     EXPECT_NOTHING,
+    EXPECT_COMMAND,
+    EXPECT_AF,
     EXPECT_BROADCAST,
     EXPECT_NETMASK,
     EXPECT_METRIC,
     EXPECT_MTU
-  } expect = EXPECT_NOTHING;
+  } expect = EXPECT_COMMAND;
 
   *ifp = parse_opt_new_ifs (argv[0]);
 
@@ -92,10 +94,37 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
 	  parse_opt_set_metric (*ifp, argv[i]);
 	  break;
 
+	case EXPECT_COMMAND:
+	  expect = EXPECT_AF;		/* Applicable at creation.  */
+	  if (!strcmp (argv[i], "create"))
+	    {
+	      error (0, 0, "interface creation is not supported");
+	      return 0;
+	    }
+	  else if (!strcmp (argv[i], "destroy"))
+	    {
+	      error (0, 0, "interface destruction is not supported");
+	      return 0;
+	    }
+	  break;
+
+	case EXPECT_AF:
 	case EXPECT_NOTHING:
 	  break;
 	}
 
+      if (expect == EXPECT_AF)	/* Address selection is single shot.  */
+	{
+	  expect = EXPECT_NOTHING;
+	  if (!strcmp (argv[i], "inet"))
+	    continue;
+	  else if (!strcmp (argv[i], "inet6"))
+	    {
+	      error (0, 0, "%s is not a supported address family", argv[i]);
+	      return 0;
+	    }
+	}
+
       if (expect != EXPECT_NOTHING)
 	expect = EXPECT_NOTHING;
       else if (!strcmp (argv[i], "broadcast"))
@@ -110,11 +139,12 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
 	parse_opt_set_flag (*ifp, IFF_UP | IFF_RUNNING, 0);
       else if (!strcmp (argv[i], "down"))
 	parse_opt_set_flag (*ifp, IFF_UP, 1);
+      else if (((mask = if_nameztoflag (argv[i], &rev))
+		& ~IU_IFF_CANTCHANGE) != 0)
+	parse_opt_set_flag (*ifp, mask, rev);
       else
 	{
-	  /* Recognize AF here.  */
-	  /* Also alias, -alias, promisc, -promisc,
-	     create, destroy, monitor, -monitor.  */
+	  /* Also alias, -alias.  */
 	  if (!((*ifp)->valid & IF_VALID_ADDR))
 	    parse_opt_set_address (*ifp, argv[i]);
 	  else if (!((*ifp)->valid & IF_VALID_DSTADDR))
@@ -140,10 +170,12 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
       error (0, 0, "option `mtu' requires an argument");
       break;
 
+    case EXPECT_AF:		/* dummy */
+    case EXPECT_COMMAND:	/* dummy */
     case EXPECT_NOTHING:
-      break;
+      return 1;
     }
-  return expect == EXPECT_NOTHING;
+  return 0;
 }
 
 int
@@ -168,6 +200,60 @@ struct ifmediareq ifm;
 
 struct if_nameindex* (*system_if_nameindex) (void) = if_nameindex;
 
+void
+system_fh_brdaddr_query (format_data_t form, int argc, char *argv[])
+{
+  ESTABLISH_IFADDRS
+  if (!ifp)
+    select_arg (form, argc, argv, 1);
+  else
+    {
+      int missing = 1;
+      struct ifaddrs *fp;
+
+      for (fp = ifp; fp; fp = fp->ifa_next)
+	{
+	  if (fp->ifa_addr->sa_family != AF_INET ||
+	      strcmp (fp->ifa_name, form->ifr->ifr_name))
+	    continue;
+
+	  if (fp->ifa_netmask)
+	    missing = 0;
+	  break;
+	}
+      select_arg (form, argc, argv, missing);
+    }
+}
+
+void
+system_fh_brdaddr (format_data_t form, int argc, char *argv[])
+{
+  ESTABLISH_IFADDRS
+  if (!ifp)
+    put_string (form, "(unknown)");
+  else
+    {
+      int missing = 1;
+      struct ifaddrs *fp;
+
+      for (fp = ifp; fp; fp = fp->ifa_next)
+	{
+	  if (fp->ifa_addr->sa_family != AF_INET ||
+	      strcmp (fp->ifa_name, form->ifr->ifr_name))
+	    continue;
+
+	  if (fp->ifa_broadaddr)
+	    {
+	      missing = 0;
+	      put_addr (form, argc, argv, fp->ifa_broadaddr);
+	    }
+	  break;
+	}
+      if (missing)
+	put_string (form, "(unknown)");
+    }
+}
+
 void
 system_fh_hwaddr_query (format_data_t form, int argc, char *argv[])
 {
@@ -372,9 +458,9 @@ system_fh_media (format_data_t form, int argc _GL_UNUSED_PARAMETER,
 	      if (item->ifmt_word == IFM_SUBTYPE (ifm.ifm_active))
 		active_subtype = item->ifmt_string;
 	    }
-	  ;;
+	  ;
 	default:
-	  ;;
+	  ;
 	}
 
       /* Shared media sub-types.  Only works if the previous
@@ -412,6 +498,60 @@ system_fh_media (format_data_t form, int argc _GL_UNUSED_PARAMETER,
     put_string (form, "(not known)");
 }
 
+void
+system_fh_netmask_query (format_data_t form, int argc, char *argv[])
+{
+  ESTABLISH_IFADDRS
+  if (!ifp)
+    select_arg (form, argc, argv, 1);
+  else
+    {
+      int missing = 1;
+      struct ifaddrs *fp;
+
+      for (fp = ifp; fp; fp = fp->ifa_next)
+	{
+	  if (fp->ifa_addr->sa_family != AF_INET ||
+	      strcmp (fp->ifa_name, form->ifr->ifr_name))
+	    continue;
+
+	  if (fp->ifa_netmask)
+	    missing = 0;
+	  break;
+	}
+      select_arg (form, argc, argv, missing);
+    }
+}
+
+void
+system_fh_netmask (format_data_t form, int argc, char *argv[])
+{
+  ESTABLISH_IFADDRS
+  if (!ifp)
+    put_string (form, "(unknown)");
+  else
+    {
+      int missing = 1;
+      struct ifaddrs *fp;
+
+      for (fp = ifp; fp; fp = fp->ifa_next)
+	{
+	  if (fp->ifa_addr->sa_family != AF_INET ||
+	      strcmp (fp->ifa_name, form->ifr->ifr_name))
+	    continue;
+
+	  if (fp->ifa_netmask)
+	    {
+	      missing = 0;
+	      put_addr (form, argc, argv, fp->ifa_netmask);
+	    }
+	  break;
+	}
+      if (missing)
+	put_string (form, "(netmask unknown)");
+    }
+}
+
 void
 system_fh_status_query (format_data_t form, int argc, char *argv[])
 {
