@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free
-  Software Foundation, Inc.
+  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
+  2014, 2015 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -645,10 +645,22 @@ setup (struct servtab *sep)
 	  close (sep->se_fd);
 	  goto tryagain;
 	}
-      if (debug)
-	fprintf (stderr, "bind failed on %s/%s: %s\n",
-		 sep->se_service, sep->se_proto, strerror (errno));
-      syslog (LOG_ERR, "%s/%s: bind: %m", sep->se_service, sep->se_proto);
+      if (sep->se_node)
+	{
+	  if (debug)
+	    fprintf (stderr, "bind failed for %s %s/%s: %s\n",
+		     sep->se_node, sep->se_service, sep->se_proto,
+		     strerror (errno));
+	  syslog (LOG_ERR,"%s %s/%s: bind: %m",
+		  sep->se_node, sep->se_service, sep->se_proto);
+	}
+      else
+	{
+	  if (debug)
+	    fprintf (stderr, "bind failed for %s/%s: %s\n",
+		     sep->se_service, sep->se_proto, strerror (errno));
+	  syslog (LOG_ERR,"%s/%s: bind: %m", sep->se_service, sep->se_proto);
+	}
       close (sep->se_fd);
       sep->se_fd = -1;
       if (!timingout)
@@ -871,8 +883,22 @@ expand_enter (struct servtab *sep)
       else
 	errmsg = gai_strerror (err);
 
-      syslog (LOG_ERR, "%s/%s: getaddrinfo: %s",
-	      sep->se_service, sep->se_proto, errmsg);
+      if (sep->se_node)
+	{
+	  if (debug)
+	    fprintf (stderr, "resolution of %s %s/%s failed: %s\n",
+		     sep->se_node, sep->se_service, sep->se_proto, errmsg);
+	  syslog (LOG_ERR, "%s %s/%s: getaddrinfo: %s",
+		  sep->se_node, sep->se_service, sep->se_proto, errmsg);
+	}
+      else
+	{
+	  if (debug)
+	    fprintf (stderr, "resolution of %s/%s failed: %s\n",
+		     sep->se_service, sep->se_proto, errmsg);
+	  syslog (LOG_ERR, "%s/%s: getaddrinfo: %s",
+		  sep->se_service, sep->se_proto, errmsg);
+	}
       return 1;
     }
 
@@ -1938,7 +1964,12 @@ main (int argc, char *argv[], char *envp[])
 
   if (!debug)
     {
-      daemon (0, 0);
+      if (daemon (0, 0) < 0)
+	{
+	  syslog (LOG_DAEMON | LOG_ERR,
+		  "%s: Unable to enter daemon mode, %m", argv[0]);
+	  exit (EXIT_FAILURE);
+	};
     }
 
   openlog ("inetd", LOG_PID | LOG_NOWAIT, LOG_DAEMON);
