@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
   2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
-  2013 Free Software Foundation, Inc.
+  2013, 2014, 2015 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -333,8 +333,9 @@ check_host (struct sockaddr *sa, socklen_t len)
 
   if (sa->sa_family != AF_INET
 # ifdef IPV6
-      && sa->sa_family != AF_INET6)
+      && sa->sa_family != AF_INET6
 # endif
+     )
     return 1;
 
 # if HAVE_DECL_GETNAMEINFO
@@ -865,7 +866,7 @@ rlogind_auth (int fd, struct auth_data *ap)
   struct hostent *hp;
   void *addrp;
 #endif
-  char *hostname;
+  char *hostname = "";
   int authenticated = 0;
   int port;
 
