@@ -1,6 +1,6 @@
 /* syslogd - log system messages
   Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-  2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free Software
+  2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Free Software
   Foundation, Inc.
 
   This file is part of GNU Inetutils.
@@ -117,7 +117,7 @@
 #endif
 
 #ifndef LOG_MAKEPRI
-#  define LOG_MAKEPRI(fac, p)	(((fac) << 3) | (p))
+#  define LOG_MAKEPRI(fac, p)	((fac) | (p))
 #endif
 
 #include <error.h>
@@ -315,6 +315,7 @@ int NoForward;			/* Don't forward messages.  */
 time_t now;			/* Time use for mark and forward supending.  */
 int force_sync;			/* GNU/Linux behaviour to sync on every line.
 				   This off by default. Set to 1 to enable.  */
+int set_local_time = 0;		/* Record local time, not message time.  */
 
 const char args_doc[] = "";
 const char doc[] = "Log system messages.";
@@ -324,7 +325,6 @@ enum {
   OPT_NO_FORWARD = 256,
   OPT_NO_KLOG,
   OPT_NO_UNIXAF,
-  OPT_PIDFILE,
   OPT_IPANY
 };
 
@@ -356,7 +356,7 @@ static struct argp_option argp_options[] = {
 #endif
   {"no-unixaf", OPT_NO_UNIXAF, NULL, 0, "do not listen on unix domain "
    "sockets (overrides -a and -p)", GRP+1},
-  {"pidfile", OPT_PIDFILE, "FILE", 0, "override pidfile (default: "
+  {"pidfile", 'P', "FILE", 0, "override pidfile (default: "
    PATH_LOGPID ")", GRP+1},
   {"rcfile", 'f', "FILE", 0, "override configuration file (default: "
    PATH_LOGCONF ")",
@@ -366,6 +366,7 @@ static struct argp_option argp_options[] = {
   {"socket", 'p', "FILE", 0, "override default unix domain socket " PATH_LOG,
    GRP+1},
   {"sync", 'S', NULL, 0, "force a file sync on every line", GRP+1},
+  {"local-time", 'T', NULL, 0, "set local time on received messages", GRP+1},
 #undef GRP
   {NULL, 0, NULL, 0, NULL, 0}
 };
@@ -446,7 +447,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       NoUnixAF = 1;
       break;
 
-    case OPT_PIDFILE:
+    case 'P':
       PidFile = arg;
       break;
 
@@ -467,6 +468,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
       force_sync = 1;
       break;
 
+    case 'T':
+      set_local_time = 1;
+      break;
+
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -1055,11 +1060,17 @@ printline (const char *hname, const char *msg)
       if (*p == '>')
 	++p;
     }
+
+  /* This overrides large positive and overflowing negative values.  */
   if (pri & ~(LOG_FACMASK | LOG_PRIMASK))
     pri = DEFUPRI;
 
-  /* don't allow users to log kernel messages */
-  if (LOG_FAC (pri) == LOG_KERN)
+  /* Avoid undefined facilities.  */
+  if (LOG_FAC (pri) > LOG_NFACILITIES)
+    pri = DEFUPRI;
+
+  /* Do not allow users to log kernel messages.  */
+  if (LOG_FAC (pri) == (LOG_KERN >> 3))
     pri = LOG_MAKEPRI (LOG_USER, LOG_PRI (pri));
 
   q = line;
@@ -1181,7 +1192,10 @@ logmsg (int pri, const char *msg, const char *from, int flags)
     timestamp = ctime (&now) + 4;
   else
     {
-      timestamp = msg;
+      if (set_local_time)
+	timestamp = ctime (&now) + 4;
+      else
+	timestamp = msg;
       msg += 16;
       msglen -= 16;
     }
@@ -1823,7 +1837,6 @@ load_conffile (const char *filename, struct filed **nextp)
 {
   FILE *cf;
   struct filed *f;
-  char *p;
 #ifndef LINE_MAX
 # define LINE_MAX 2048
 #endif
@@ -1882,6 +1895,7 @@ load_conffile (const char *filename, struct filed **nextp)
      - deal with continuation lines, last char is '\' .  */
   while (fgets (cline, line_max - (cline - cbuf), cf) != NULL)
     {
+      char *p;
       size_t len = strlen (cline);
 
       /* If this is a continuation line, skip leading whitespace for
@@ -1975,7 +1989,7 @@ load_conffile (const char *filename, struct filed **nextp)
       if (*p == '\0' || *p == '#')
 	continue;
 
-      strcpy (cline, p);
+      memmove (cline, p, strlen (p) + 1);
 
       /* Cut the trailing spaces.  */
       for (p = strchr (cline, '\0'); isspace (*--p);)
@@ -2234,7 +2248,7 @@ cfline (const char *line, struct filed *f)
       else
 	{
 	  pri = decode (bp, prioritynames);
-	  if (pri < 0)
+	  if (pri < 0 || (pri > LOG_PRIMASK && pri != INTERNAL_NOPRI))
 	    {
 	      snprintf (ebuf, sizeof (ebuf),
 			"unknown priority name \"%s\"", bp);
@@ -2280,9 +2294,8 @@ cfline (const char *line, struct filed *f)
 	  else
 	    {
 	      i = decode (buf, facilitynames);
-	      facilities_seen |= (1 << LOG_FAC (i));
 
-	      if (i < 0)
+	      if (i < 0 || i > (LOG_NFACILITIES << 3))
 		{
 		  snprintf (ebuf, sizeof (ebuf),
 			    "unknown facility name \"%s\"", buf);
@@ -2292,6 +2305,8 @@ cfline (const char *line, struct filed *f)
 
 	      f->f_pmask[LOG_FAC (i)] &= ~pri_clear;
 	      f->f_pmask[LOG_FAC (i)] |= pri_set;
+
+	      facilities_seen |= (1 << LOG_FAC (i));
 	    }
 	  while (*p == ',' || *p == ' ')
 	    p++;
