@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
   2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
-  2013 Free Software Foundation, Inc.
+  2013, 2014, 2015 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -1015,9 +1015,7 @@ dooption (int option)
 	case TELOPT_TSPEED:
 	case TELOPT_LFLOW:
 	case TELOPT_XDISPLOC:
-#ifdef	TELOPT_ENVIRON
 	case TELOPT_NEW_ENVIRON:
-#endif
 	case TELOPT_OLD_ENVIRON:
 	default:
 	  break;
@@ -1317,9 +1315,7 @@ suboption (void)
 	break;
       }				/* end of case TELOPT_XDISPLOC */
 
-#ifdef	TELOPT_NEW_ENVIRON
     case TELOPT_NEW_ENVIRON:
-#endif
     case TELOPT_OLD_ENVIRON:
       {
 	register int c;
@@ -1338,7 +1334,6 @@ suboption (void)
 	else if (c != TELQUAL_INFO)
 	  return;
 
-#ifdef	TELOPT_NEW_ENVIRON
 	if (subchar == TELOPT_NEW_ENVIRON)
 	  {
 	    while (!SB_EOF ())
@@ -1349,7 +1344,6 @@ suboption (void)
 	      }
 	  }
 	else
-#endif
 	  {
 #ifdef	ENV_HACK
 	    /*
@@ -1702,7 +1696,9 @@ send_status (void)
   net_output_datalen (statusbuf, ncp - statusbuf);
   netflush ();			/* Send it on its way */
 
-  DEBUG (debug_options, 1, printsub ('>', statusbuf, ncp - statusbuf));
+  /* Step over the initial IAC+SB, into suboption payload.  */
+  DEBUG (debug_options, 1, printsub ('>', statusbuf + 2,
+				     ncp - statusbuf - 2));
   return;
 
 trunc:
