@@ -1,6 +1,6 @@
 #!/bin/sh
 
-# Copyright (C) 2011, 2012, 2013 Free Software Foundation, Inc.
+# Copyright (C) 2011, 2012, 2013, 2014, 2015 Free Software Foundation, Inc.
 #
 # This file is part of GNU Inetutils.
 #
@@ -250,6 +250,10 @@ fi
 
 cat <<EOT > "$TMPDIR/inetd.conf"
 $PORT stream tcp4 nowait $USER $PWD/$FTPD ftpd -A -l
+EOT
+
+test "$TEST_IPV6" = "no" ||
+    cat <<EOT >> "$TMPDIR/inetd.conf"
 $PORT stream tcp6 nowait $USER $PWD/$FTPD ftpd -A -l
 EOT
 
@@ -260,8 +264,6 @@ fi
 
 cat <<EOT > "$TMPDIR/.netrc"
 machine $TARGET login $FTPUSER password foobar
-machine $TARGET6 login $FTPUSER password foobar
-machine $TARGET46 login $FTPUSER password foobar
 EOT
 
 if test $? -ne 0; then
@@ -269,6 +271,13 @@ if test $? -ne 0; then
     exit 77
 fi
 
+if test "$TEST_IPV6" != "no"; then
+    cat <<-EOT >> "$TMPDIR/.netrc"
+	machine $TARGET6 login $FTPUSER password foobar
+	machine $TARGET46 login $FTPUSER password foobar
+	EOT
+fi
+
 chmod 600 "$TMPDIR/.netrc"
 
 # Some simple, but variable content.
@@ -393,6 +402,37 @@ HOME=$TMPDIR $FTP "$TARGET" $PORT -4 -v -p -t >$TMPDIR/ftp.stdout 2>&1
 
 test_report $? "$TMPDIR/ftp.stdout" "EPSV/$TARGET"
 
+# Test a passive connection: EPSV and IPv4.
+#
+# Set NETRC in environment to regulate login.
+#
+echo "EPSV to $TARGET (IPv4) using inetd, setting NETRC."
+cat <<STOP |
+rstatus
+epsv4
+dir
+`$do_transfer && test -n "$DLDIR" && echo "\
+cd $DLDIR"`
+`$do_transfer && echo "\
+lcd $TMPDIR
+image
+put $GETME $PUTME"`
+STOP
+
+NETRC=$TMPDIR/.netrc \
+  $FTP "$TARGET" $PORT -4 -v -p -t >$TMPDIR/ftp.stdout 2>&1
+
+test_report $? "$TMPDIR/ftp.stdout" "EPSV/$TARGET with NETRC"
+
+$do_transfer && \
+    if cmp -s "$TMPDIR/$GETME" "$FTPHOME$DLDIR/$PUTME"; then
+	test "${VERBOSE+yes}" && echo >&2 'Binary transfer succeeded.'
+	date "+%s" >> "$TMPDIR/$GETME"
+    else
+	echo >&2 'Binary transfer failed.'
+	exit 1
+    fi
+
 # Test an active connection: EPRT and IPv4.
 #
 echo "EPRT to $TARGET (IPv4) using inetd."
@@ -420,35 +460,51 @@ $do_transfer && \
 	exit 1
     fi
 
-# Test a passive connection: EPSV and IPv6.
+# Test an active connection: EPRT and IPv4.
 #
-echo "EPSV to $TARGET6 (IPv6) using inetd."
+# Use `-N' to set location of .netrc file.
+#
+echo "EPRT to $TARGET (IPv4) using inetd, apply the switch -N."
 cat <<STOP |
 rstatus
+epsv4
 dir
 STOP
-HOME=$TMPDIR $FTP "$TARGET6" $PORT -6 -v -p -t >$TMPDIR/ftp.stdout 2>&1
 
-test_report $? "$TMPDIR/ftp.stdout" "EPSV/$TARGET6"
+$FTP "$TARGET" $PORT -N"$TMPDIR/.netrc" -4 -v -t >$TMPDIR/ftp.stdout 2>&1
 
-# Test an active connection: EPRT and IPv6.
-#
-echo "EPRT to $TARGET6 (IPv6) using inetd."
-cat <<STOP |
-rstatus
-dir
-`$do_transfer && test -n "$DLDIR" && echo "\
+test_report $? "$TMPDIR/ftp.stdout" "EPRT/$TARGET"
+
+if test "$TEST_IPV6" != "no" && test -n "$TARGET6"; then
+    # Test a passive connection: EPSV and IPv6.
+    #
+    echo "EPSV to $TARGET6 (IPv6) using inetd."
+    cat <<-STOP |
+	rstatus
+	dir
+	STOP
+    HOME=$TMPDIR $FTP "$TARGET6" $PORT -6 -v -p -t >$TMPDIR/ftp.stdout 2>&1
+
+    test_report $? "$TMPDIR/ftp.stdout" "EPSV/$TARGET6"
+
+    # Test an active connection: EPRT and IPv6.
+    #
+    echo "EPRT to $TARGET6 (IPv6) using inetd."
+    cat <<-STOP |
+	rstatus
+	dir
+	`$do_transfer && test -n "$DLDIR" && echo "\
 cd $DLDIR"`
-`$do_transfer && echo "\
+	`$do_transfer && echo "\
 lcd $TMPDIR
 image
 put $GETME $PUTME"`
-STOP
-HOME=$TMPDIR $FTP "$TARGET6" $PORT -6 -v -t >$TMPDIR/ftp.stdout 2>&1
+	STOP
+    HOME=$TMPDIR $FTP "$TARGET6" $PORT -6 -v -t >$TMPDIR/ftp.stdout 2>&1
 
-test_report $? "$TMPDIR/ftp.stdout" "EPRT/$TARGET6"
+    test_report $? "$TMPDIR/ftp.stdout" "EPRT/$TARGET6"
 
-$do_transfer && \
+    $do_transfer && \
     if cmp -s "$TMPDIR/$GETME" "$FTPHOME$DLDIR/$PUTME"; then
 	test "${VERBOSE+yes}" && echo >&2 'Binary transfer succeeded.'
 	date "+%s" >> "$TMPDIR/$GETME"
@@ -456,6 +512,7 @@ $do_transfer && \
 	echo >&2 'Binary transfer failed.'
 	exit 1
     fi
+fi # TEST_IPV6
 
 # Availability of IPv4-mapped IPv6 addresses.
 #
@@ -510,7 +567,8 @@ fi
 
 # Test functionality of IPv4-mapped IPv6 addresses.
 #
-if $have_address_mapping && test -n "$TARGET46" ; then
+if $have_address_mapping && test -n "$TARGET46" &&
+   test "$TEST_IPV6" != "no"; then
     # Test a passive connection: EPSV and IPv4-mapped-IPv6.
     #
     echo "EPSV to $TARGET46 (IPv4-as-IPv6) using inetd."
@@ -564,7 +622,7 @@ put $GETME $PUTME"`
 else
     # The IPv4-as-IPv6 tests were not performed.
     echo 'Skipping two tests of IPv4 mapped as IPv6.'
-fi
+fi # have_address_mapping && TEST_IPV6
 
 # Test name mapping with PASV and IPv4.
 # Needs a writable destination!
