@@ -1,6 +1,6 @@
 #!/bin/sh
 
-# Copyright (C) 2011, 2012, 2013 Free Software Foundation, Inc.
+# Copyright (C) 2011, 2012, 2013, 2014, 2015 Free Software Foundation, Inc.
 #
 # This file is part of GNU Inetutils.
 #
@@ -21,10 +21,12 @@
 #
 #  * Shell: SVR3 Bourne shell, or newer.
 #
-#  * id(1), uname(1).
+#  * id(1), mktemp(1), uname(1).
 
 . ./tools.sh
 
+$need_mktemp || exit_no_mktemp
+
 hostname=${hostname:-../src/hostname$EXEEXT}
 
 if [ $VERBOSE ]; then
@@ -34,22 +36,43 @@ fi
 
 errno=0
 
+posttest () {
+    test -n "$NAMEFILE" && test -r "$NAMEFILE" && rm "$NAMEFILE"
+}
+
 $hostname || errno=$?
 test $errno -eq 0 || echo "Failed to get hostname." >&2
 test $errno -eq 0 || exit $errno
 
 test `$hostname` = `uname -n` || errno=$?
-test $errno -eq 0 || echo "Failed to get hostname." >&2
+test $errno -eq 0 || echo "Failed to get same hostname as uname does." >&2
 test $errno -eq 0 || exit $errno
 
-if [ `func_id_uid` != 0 ]; then
+if test `func_id_uid` != 0; then
     echo "hostname: skipping tests to set host name"
 else
     # Only run this if hostname succeeded...
-    if [ $errno -eq 0 ]; then
+    if test $errno -eq 0; then
 	$hostname `$hostname` || errno=$?
 	test $errno -eq 0 || echo "Failed to set hostname." >&2
 	test $errno -eq 0 || exit $errno
+
+	NAMEFILE=`$MKTEMP tmp.XXXXXXXXXX` || errno=$?
+	test $errno -eq 0 || echo >&2 'Cannot create test file.'
+	test $errno -eq 0 || exit $errno
+
+	trap posttest 0 1 2 3 15
+
+	SAVEDNAME=`hostname` || errno=$?
+	echo $SAVEDNAME > $NAMEFILE
+
+	if test $errno -eq 0 && test -s $NAMEFILE; then
+	    $hostname -F $NAMEFILE || errno=$?
+	    test $errno -eq 0 || echo >&2 'Failed to set hostname.'
+
+	    # Attempt to rescue name using the first method.
+	    test $errno -eq 0 || $hostname "$SAVEDNAME" || errno=$?
+	fi
     fi
 fi
 
